/*! \mainpage Overview
 *
 * \section intro_sec Introduction
 *
 * This document outlines the structures and functions which make up the SIMPHYNITY game plugin example DLL.
 * The supplied Microsoft Visual Studio 2008 workspace is preconfigured to build the example.
 *
 * \section sf_sec Supported Functionality
 *
 * The provided template demonstrates the use of all functions required to configure / use the DLL.  The structures
 * and functions required are all outlined in the dllmain.h documentation (in the 'Files' section), and allow
 * you to specify the default game settings, SIMPHYNITY remote configuration and game telemetry data each frame, as
 * well as respond to SIMPHYNITY loaded / unloaded and game detected / lost events.
 *
 * \section install_sec Installation
 *
 * Once compiled, the target DLL should be renamed to that of the game title. This is the title string that
 * will appear in SIMPHYNITY to identify the game (remember to leave the .dll extension in place).
 * The DLL must be placed in the 'gameplugins' subdirectory of the main SIMPHYNITY installation directory, and
 * <b>should not exceed 16 characters</b>.
 */

/** \file dllmain.h
 * Contains declarations for use by the SIMPHYNITY plugin DLL.
 * NB The compiled .dll file <b>must be placed in the <i>gameplugins</i> subdirectory in SIMPHYNITY's installation directory, and
 * named after the game it pertains to</b>, as this will be assumed to be the game 'title' in SIMPHYNITY. 
 *
 */

#define DLL_EXPORT

#if defined DLL_EXPORT
	#define DECLDIR __declspec(dllexport)
#else
	#define DECLDIR __declspec(dllimport)
#endif

/**
 * Display speed units (for use by <i>GAME_SETTINGS.displaySpeedUnits</i>).
 *
 * Members:-
 * 
 * <b>DISPLAY_SPEED_UNITS_KPH</b>
 *
 *  Show display speed as kilometers per hour (KPH).
 * 
 * <b>DISPLAY_SPEED_UNITS_MPH</b>
 *
 *  Show display speed as miles per hour (MPH).
 *
 * <b>DISPLAY_SPEED_UNITS_KNOTS</b>
 *
 *  Show display speed as knots (Kts).
 *
 * <b>DISPLAY_SPEED_UNITS_MS</b>
 *
 *  Show display speed as raw meters per second (m/s).  Dial will not have units type displayed.
 */
enum DISPLAY_SPEED_UNITS
{
	DISPLAY_SPEED_UNITS_KPH = 0,
	DISPLAY_SPEED_UNITS_MPH,
	DISPLAY_SPEED_UNITS_KNOTS,
	DISPLAY_SPEED_UNITS_MS
};

/**
 * Contains default settings for the game (as configurable via the 'game settings' panel
 * within the Simphynity UI).  NB these are not to be confused with user-specific settings (which
 * pertain to output devices), rather global settings for this game.
 *
 * Members:-
 * 
 * <b>float maxGs</b>
 *
 *  This tells SIMPHYNITY the level of force that is considered to be '100 percent'. E.g. if this was set to 
 *  1.0, and the current game output is +1.0 G's on the longitudinal (pitch) axis, the simulator will be at 
 *  its furthest physical +ve pitch extent (assuming the Output Mode is 'Linear' and the 'Pitch Multiplier' 
 *  is set to 1.0 in the user's motion device 'Movement Behavior' settings).
 *
 * <b>float maxSpeed</b>
 *  
 *  As with maxGs, this tells SIMPHYNITY the vehicle speed that is considered to be '100 percent' (primarily for use 
 *  of any attached wind systems), and is given in the units specified by the 'displaySpeedUnits' member described below. 
 *  E.g. if the display speed units are DISPLAY_SPEED_UNITS_MPH and this was set to 200.0, and the current game vehicle 
 *  speed is 200 Mph, the wind simulator fans will be at 100% velocity.  NB this member is ignored if 'learnSpeed' is set
 *  to 'true'.
 *
 * <b>DISPLAY_SPEED_UNITS displaySpeedUnits</b>
 *  
 *  Denotes which speed units (e.g. MPH, KPH) are displayed in the SIMPHYNITY user interface for this game 
 * (see enum DISPLAY_SPEED_UNITS).
 *
 * <b>bool learnSpeed</b>
 *
 *	If set to 'true', the maximum speed is auto-detected - whatever it settles at is then assumed to be '100 percent' 
 *  (see also 'maxSpeed' notes above).
 *
 * <b>int accelScalar</b>
 * 
 *  Range 0 > 100 (%) 
 *  Tells SIMPHYNITY the level of acceleration forces (as a result of velocity changes) to apply to the motion output, 
 *  in addition to normal pitch and roll felt when the vehicle is tilted. This is useful for high-G applications such as 
 *  fighter-jet games, where the user might want to feel the pitch / roll of the aircraft (i.e. the sensation of normal world 
 *  gravity) at all times, while leaving a margin for acceleration forces; in this instance the 'maxGs' would likely 
 *  be set fairly low (e.g. 2.0), and the accelScalar very low (e.g. 5%).
 *  
 *  NB To treat all forces (world gravity and acceleration gravity) equally, and prevent the user being able to change this
 *  in the game settings panel, set this value to -1.
 */
struct GAME_SETTINGS
{
	float					maxGs;
	float					maxSpeed;
	DISPLAY_SPEED_UNITS		displaySpeedUnits;
	bool					learnSpeed;
	int						accelScalar;
};

/**
 * Rendering function identifiers (for use by <i>REMOTE_CONFIG.gfxRenderFunction</i>)
 *
 * Members:-
 * 
 * <b>RENDER_FUNCTION_GDI_SWAPBUFFERS</b>
 *
 *  SwapBuffers (Gdi32.dll) - default for most openGL games.
 *
 * <b>RENDER_FUNCTION_OGL_WGLSWAPBUFFERS</b>
 * 
 *	wglSwapBuffers (OpenGL32.dll)
 *   
 * <b>RENDER_FUNCTION_OGL_FINISH</b>
 *
 *	glFinish (OpenGL32.dll)
 *
 * <b>RENDER_FUNCTION_OGL_FLUSH</b>
 *
 *  glFlush (OpenGL32.dll)
 *
 * <b>RENDER_FUNCTION_D3D_PRESENT</b>
 *
 *  IDirect3DDevice8/9::Present - default for most DX8/DX9 games.
 *
 * <b>RENDER_FUNCTION_D3D_ENDSCENE</b>
 *
 *  IDirect3DDevice8/9::EndScene
 *
 * <b>RENDER_FUNCTION_D3DSC_PRESENT</b>
 *
 *  IDirect3DSwapChain9::Present
 *
 * <b>RENDER_FUNCTION_DXGI_PRESENT</b>
 *
 *  IDXGISwapChain::Present - default for most DX10/DX11 games.
 */
enum GFX_RENDER_FUNCTION
{
	// OGL
	RENDER_FUNCTION_GDI_SWAPBUFFERS = 0, // OGL default.
	RENDER_FUNCTION_OGL_WGLSWAPBUFFERS,
	RENDER_FUNCTION_OGL_FINISH,
	RENDER_FUNCTION_OGL_FLUSH,

	// DX8 / DX9
	RENDER_FUNCTION_D3D_PRESENT = 0, // DX8 / DX9 default.
	RENDER_FUNCTION_D3D_ENDSCENE,
	RENDER_FUNCTION_D3DSC_PRESENT,

	// DX10 / DX11
	RENDER_FUNCTION_DXGI_PRESENT = 0 // DX10 DEFAULT.
};

/**
 * Keyboard hooking methods (for use by <i>REMOTE_CONFIG.kbHookingMethod</i>).
 *
 * Members:-
 * 
 * <b>KEYBOARD_HOOK_METHOD_WNDPROC</b>
 *
 *  Override the process parent window's WndProc method to intercept WM_KEYDOWN messages (default best method for most games).
 *
 * <b>KEYBOARD_HOOK_METHOD_SETWINDOWSHOOKEX</b>
 *
 *  Hook the keyboard via SetWindowsHookEx (WH_KEYBOARD).
 *
 * <b>KEYBOARD_HOOK_METHOD_GETASYNCKEYSTATE</b>
 *
 *  Most brute-force approach - poll pressed state of keys we're interested in via GetAsyncKeyState method.
 */
enum KEYBOARD_HOOK_METHOD
{
	KEYBOARD_HOOK_METHOD_WNDPROC = 0,
	KEYBOARD_HOOK_METHOD_SETWINDOWSHOOKEX, 
	KEYBOARD_HOOK_METHOD_GETASYNCKEYSTATE
};

/**
 * Contains Simphynity Remote configuration settings, as supplied to the <i>GetSimphyintyRemoteSettings</i>
 * function.
 *
 * Members:-
 * 
 * <b>bool enabled</b>
 *
 *  Instructs Simphynity whether to attempt to inject Simphynity Remote into the game process.  If
 *  set to true (default), and the user has also selected the 'inject remote / telemetry plugin'
 *  option in the game settings panel of the Simphynity UI, remote injection will be attempted.
 *  NB Simphynity remote is only supported within DX8/9/10/11 and OGL based games.
 *
 * <b>GFX_RENDER_FUNCTION gfxRenderFunction</b>
 * 
 *  Assuming Simphynity remote is enabled for this game, this member indicates which last-stage drawing 
 *  function is used by the game's GFX library (see enum GFX_RENDER_FUNCTION). Simphynity remote will 
 *  attempt to hook this function and perform its drawing here, immediately after the game's frame buffer 
 *  has been rendered.
 *  IMPORTANT: The indicated GFX render function must be correct or Simphyinty Remote will fail to display.
 *  NB: A value of 0 indiciates to use the default (most common) render function for whichever GFX library
 *  is detected.
 *
 * <b>KEYBOARD_HOOK_METHOD kbHookingMethod</b>
 *
 *  Depending on how the game is configured to respond to key input, it may be necessary to use a different
 *  keyboard hooking method in order to capture key presses for use by Simphyinty remote.  The various hooking
 *  methods available are listed in the KEYBOARD_HOOK_METHOD enum.
 *  IMPORTANT: The indicated keyboard hook function must be correct or Simphyinty Remote will to 
 *  respond to keyboard input.
 *
 * <b>bool invertZPlane</b>
 *  
 *  Inverts the vertex-winding rule used to render the remote overlay.  In some games the Z (depth) axis moves
 *  away from the camera, in others it moves towards it.  If Simphynity remote is not visible, it is possible it
 *  is facing in the wrong direction to the camera - try inverting its z-plane by setting this member to 'true'.
 * 
 * <b>bool forceScreenRefresh</b>
 *
 *  Some games only refresh the screen when its contents (i.e. the frame buffer's contents) have changed.  When using
 *  Simphynity remote, we need to ensure the screen is forcibly updated whenever the remote is shown or hidden, or interacted
 *  with in any way by the user.  If the game engine does not forcibly refresh the screen each frame (irrespective of
 *  whether its contents have changed or not), we must instruct it to do so whenever Simphynity remote is displayed (by
 *  setting this member to 'true').  If the game already forces refreshes each frame, we must set this member to 'false',
 *  otherwise Simphyinty Remote will be forcing a second unecessary refresh which can cause undesirable graphical side 
 *  effects such as black 'flashes').  NB if set to 'true', refreshes will only be forced while Simphyinty Remote is activity
 *  displayed on the screen.
 */
struct REMOTE_CONFIG
{
	bool					enabled;
	GFX_RENDER_FUNCTION		gfxRenderFunction;
	KEYBOARD_HOOK_METHOD	kbHookingMethod;
	bool					invertZPlane;
	bool					forceScreenRefresh;
};

/**
 * Represents a single packet of telemetry data.  Simphynity calls the <i>GetTelemetryData</i> function
 * to update the output of any connected devices.  <i>GetTelemetryData</i> receives a TELEMETRY_DATA
 * structure as its parameter which must be populated.
 *
 * For convenience, the values required can be <i>either</i> the longitudinal, lateral and vertical local acceleration 
 * and local velocity values (set <b>useLocalVals</b> to 'true'), <i>or</i> the global velocity values accompanied by the 
 * vehicle rotation matrix relative to global co-ordinates (set <b>useLocalVals</b> to 'false').  In the case of the former, 
 * the local acceleration must include world gravity.  In the case of the latter, the acceleration is derived automatically 
 * by Simphynity from the global velocity (folded into the rotationMatrix) and packetTime.
 *
 * Simphynity will automatically detect the game output as 'idle' when no change in telemetry has been detected for a fixed
 * amount of time.
 *
 * Members:-
 * 
 * <b>bool useLocalVals</b>
 *
 *  Instructs Simphynity to use the local values specified in <b>localAccel</b> and <b>localVel</b>.
 *  Any values specified in <b>globalVel</b> or <b>rotationMatrix</b> will be ignored.
 *  Similarly, if set to 'false', any values in <b>localAccel</b> or <b>localVel</b> will be ignored.
 *  While the ability to simply specify local longitudinal, lateral and vertical acceleration (3-dof)
 *  is offered for simplicity, the advantage of using global values is it will allow Simphynity to 
 *  derive up to 6-dof of motion data should future hardware devices support it.
 *
 * <b>float localAccel[3]</b>
 *
 *  The longitudinal[0] (-ve = accelerating forward/leaning backwards), lateral[1] (-ve = accelerating right/leaning left) and vertical[2] (+ve = down) 
 *  acceleration values local to the vehicle's co-ordinate system, in M/S/S (meters per second per second).
 *  <b>Must include world gravity.</b>
 *  Only required if <b>useLocalVals</b> is set to 'true'.
 *	
 * <b>float localVel[3]</b>
 *
 *  The longitudinal[0] (+ve = forward), lateral[1] (+ve = right) and vertical[2] (+ve = down) 
 *  velocity values local to the vehicle's co-ordinate system, in M/S (meters per second).
 *  Only required if <b>useLocalVals</b> is set to 'true'.
 *	
 * <b>float globalVel[3]</b>
 *
 *  The X[0], Y[1] and Z[2] velocity values of the vehicle relative to the world (global) 
 *  co-ordinate system, in M/S (meters per second).  
 *  Only required if <b>useLocalVals</b> is set to 'false'.
 *
 * <b>float rotationMatrix[3]</b>
 *
 *  The rotation matrix representing the rotation of the vehicle relative to the world (global) co-ordinate system.
 *  Only required if <b>useLocalVals</b> is set to 'false'.
 *
 * <b>DWORD packetTimeMillis</b>
 *
 *  The world (game) time this packet pertains to, in milliseconds.  Must be the exact time of the frame as represented 
 *  by the physics engine of the game (as it is used to calculate acceleration values from changes in velocity).
 *  Only required if <b>useLocalVals</b> is set to 'false'.
 */
struct TELEMETRY_DATA
{
	bool useLocalVals;
	float localAccel[3];
    float localVel[3];
    float globalVel[3];
	float rotationMatrix[3][3];
	DWORD packetTimeMillis;
};


/**
 * Entry point for the DLL.  Perform any initialization / destruction required for this plugin here.
 * (See Win32 documentation for more information).
 */
BOOL APIENTRY DllMain(HMODULE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved);

extern "C"
{
	/************************************* DEFAULT GAME SETTINGS **************************************/

	/**
	 * Configures the default settings for this game (as configurable by the game settings panel in
	 * the Simphynity UI).  It is mandatory this structure is correctly configured - for more information
	 * see the GAME_SETTINGS structure documentation.
	 *
	 * @param settings The game settings structure.
	 */
	DECLDIR void GetDefaultSettings(const GAME_SETTINGS* settings);


	/******************************************** PROCESS INFO ********************************************/

	/**
	 * Called by Simphynity to query the number of possible executable names that exist for this game.  E.g.
	 * some titles might have separate dx9 and/or dx11 executables with different names.  It is the executable
	 * name (combined with the parent dirs, see below) that Simphynity uses to determine if the game is running.
	 *
	 * @return The number of executable names (N).  Simphyinty will then call 'GetExeName' N times passing the
	 *		   index of the executable name requested, along with a buffer to copy the name to (see GetExeName(...))
	 *		   Range must be 1 > N
	 */
	DECLDIR int GetNumExeNames();

	/**
	 * Called by Simphynity to get executable name N for the game (specified by the supplied index).  See 
	 * also GetNumExeNames(...).
	 * 
	 * @param index The index of the executable name requested
	 * @param exeName The buffer to copy the (null-terminated) executable name string into.
	 *				  <b>NB: MUST NOT EXCEED 32 CHARACTERS</b>.
	 */
	DECLDIR void GetExeName(int index, const TCHAR* exeName);

	/**
	 * Called by Simphynity to query the number of possible parent sub-directories that exist for this game's
	 * executable (.exe) file.  E.g. some titles may be installed via different methods (e.g. digital download
	 * or DVD), and installed to different paths depending on the method.  This gives the possible most
	 * deeply-nested install subpaths (not fully qualified paths) for the executable file.
	 *
	 * The purpose of this information is to allow Simphynity to better determine that it has detected the
	 * game we're actually interested in, and not another process with the same executable name.  When Simphynity
	 * is idle, it periodically checks all running processes and gets their full installation paths.  If the
	 * executable name (excluding its path) is matched with one of those returned from 'GetExeName(...)' above, 
	 * it then proceeds to check the immediate parent directory (or path) of the executable to further determine
	 * this is definately the .exe we're interested in.  A parent directory can be specified in as many depth levels
	 * (starting from the exe name and working upwards through the directory hierarchy) as desired, each depth
	 * level being separated via a '/' (or '\\', NB '\' will not function as intended)
	 *
	 * E.g. 'rFactor.exe' may have the parent directory "bin", or if we want Simphynity to be more scrutinous,
	 *		"rFactor/bin".
	 *
	 * If we don't wish to specify parent dirs (and detect purely by executable name alone), simply return '0' from
	 * GetNumParentDirs().
	 *
	 * @return The number of possible parent directories, or 0 to detect only by executable name.
	 *		   Range 0 > N
	 */
	DECLDIR int GetNumParentDirs();

	/**
	 * Called by Simphynity to get possible parent directory N for the game (specified by the supplied index).  See 
	 * GetNumParentDirs() for more information.
	 * 
	 * @param index The index of the parent directory requested
	 * @param exeName The buffer to copy the (null-terminated) parent directory string into.
	 *				  <b>NB: MUST NOT EXCEED 128 CHARACTERS</b>.
	 */
	DECLDIR void GetParentDir(int index, const TCHAR* dir);


	/********************************* SIMPHYNITY REMOTE SETTINGS **********************************/

	/**
	 * Supplied with a REMOTE_CONFIG parameter (containing pre-configured default values) to 
	 * allow Simphynity Remote to be enabled/disabled and/or configured according to our requirements.  
	 * See the REMOTE_CONFIG structure documentation for more information regarding its members.
	 *
	 * @param settings The Simphynity Remote settings for this game.
	 */
	DECLDIR void GetSimphyintyRemoteSettings(const REMOTE_CONFIG* settings);


	/******************************************** EVENTS ********************************************/

	/**
	 * Called by Simphynity when our game process has been detected.  
	 * Perform any actions required upon game detection here.
	 */
	DECLDIR void OnGameDetected();

	/**
	 * Called by Simphynity when our game process has been lost (i.e. the game
	 * has been terminated).  Perform any actions required upon game undetection here.
	 */
	DECLDIR void OnGameLost();

	/**
	 * Called by Simphynity to request the most recent telemetry data available from the game.
	 * This is the data used to update any in-game output dials, device output previews and the
	 * devices themselves.  See the TELEMETRY_DATA structure documentation for more information
	 * regarding what needs to be provided here.
	 *
	 * @param telem The structure to update with the latest telemetry data.
	 */
	DECLDIR void GetTelemetryData(const TELEMETRY_DATA* telem);
}

